import sys
import QUB
import QUBB
import Reports
import threading
import os.path
import string
import heapq
import traceback
import time

from math import *

from HistogramCommands import MakeHistogramSheet

import win32com.client
from pythoncom import CoInitialize, CoUninitialize



CoInitialize()


def setNbox(cell, val):
    cell.Value = val
    cell.Borders.LineStyle = 1 # xlContinuous

class ExcelReport:
    def __init__(self):
        self.excel = win32com.client.Dispatch("Excel.Application")
        self.book = self.excel.Workbooks.Add(-4167) # XLWBatWorksheet
        self.scrap = self.book.Worksheets.Item(1)
        self.scrap.Cells.HorizontalAlignment = -4108 # 0xffffeff4 # xlHAlignCenter

        self.excel.Visible = 1

    def writeTable(self, tbl, embed=0):
        sheet = self.book.Sheets.Add()
        cells = sheet.Cells
        maxcol = 0
        for row in tbl:
            if maxcol < len(row):
                maxcol = len(row)

        for i in range(0, len(tbl)):
            for j in range(0, len(tbl[i])):
                setNbox( cells(i+1, j+1), tbl[i][j] )
            for j in range(len(tbl[i]), maxcol):
                setNbox(cells(i+1, j+1), '')

        return sheet

    def writeHistogram(self, hist, group_title, w=300, h=200):
        sheet = self.book.Sheets.Add()
        chart = MakeHistogramSheet(hist, sheet, 1, int(w*1.4), int(h*1.4))
        chart.Width = w
        chart.Height = h
        sheet.Cells(1, 2).Value = group_title

        return sheet, chart



def EnsureExcelReport(context):
    try:
        win32com.client.gencache.EnsureModule('{00020813-0000-0000-C000-000000000046}',0,1,3)
    except:
        traceback.print_exc()

    try:
        return context.ExcelReport
    except AttributeError:
        context.ExcelReport = ExcelReport()
        return context.ExcelReport


def ReportHistograms(results, context):
    rep = EnsureExcelReport(context)
    node = results.node

    def ReportHistGroup(group):
        group_title = str(group['Title'].data)
        hist = group.find('Histogram')
        while not hist.isNull:
            rep.writeHistogram(hist, group_title, 300, 200)
            hist = hist.nextSameName()

    ApplyToAllByName(node, 'HistogramGroup', ReportHistGroup)        

def ApplyToAllByName(root, name, func):
    if root.name == name:
        func(root)
    child = root.child
    while not child.isNull:
        ApplyToAllByName(child, name, func)
        child = child.sibling

Reports.Add('Histograms to Excel', ReportHistograms, ['MIL', 'MPL', 'MSearch', 'SKM', 'AMP', 'Stat', 'IV', 'ChopIdl', 'IdlBase'])
